"""
    ProductList Patch

This patch allows you to select that certain products
(Actually meta_types) do not get listed in the
"Select Type To Add..." box

The page to select the hidden products is added to 'Folder'
as a new tab.
"""
__version__ = "4.1.0"

import OFS.Folder
import App.ApplicationManager
import Globals
import string
from DateTime import DateTime
from Acquisition import aq_acquire

def all_meta_types(self, REQUEST=None):
    """ Get a filtered list of products """
    if not REQUEST:
        REQUEST = getattr(self, 'REQUEST', None)
    newList = []
    list = self.unfiltered_meta_types()
    exclude = getGlobalExclude(self)
    if REQUEST:
        exclude.extend(getPersonalExclude(self, REQUEST))
    include = getPersonalInclude(self, REQUEST)
    for item in list:
        if item['name'] not in exclude or item['name'] in include:
            newList.append(item)
    return newList

def getGlobalExclude(self):
    """ Get the Global Exclude object """
    cp = aq_acquire(self, 'Control_Panel')
    geo = getattr(cp, 'geo', [])
    return geo

def setGlobalExclude(self, geo=[]):
    """ Return the global exclusions list """
    cp = aq_acquire(self, 'Control_Panel')
    setattr(cp, 'geo', geo)

def getPersonalExclude(self, REQUEST):
    """ Return the contents of the users exclusions cookie """
    list = []
    if REQUEST.cookies.has_key('DontListProducts'):
        list = REQUEST.cookies['DontListProducts']
        list = map(lambda x: string.strip(x)[1:-1], string.split(list[1:-1], ','))
    return list

def getPersonalInclude(self, REQUEST):
    """ Return the contents of the users exclusions cookie """
    list = []
    if REQUEST.cookies.has_key('DoListProducts'):
        list = REQUEST.cookies['DoListProducts']
        list = map(lambda x: string.strip(x)[1:-1], string.split(list[1:-1], ','))
    return list

def manage_hideProducts(self, REQUEST, RESPONSE):
    """ Reads the list of products, then checks if it should be
        hidden, sets a cookie and redirects """
    if REQUEST['AUTHENTICATED_USER'].has_role(['Manager']):
        # set global stuff
        gList = []
        for item in self.unfiltered_meta_types():
            if REQUEST.get('cbg_' + string.replace(item['name'], ' ', '_'), 0):
                gList.append(item['name'])
        setGlobalExclude(self, gList)
    # set personal stuff
    d = DateTime() + 365
    if id!='Control_Panel':
        pList = []
        for item in self.unfiltered_meta_types():
            if REQUEST.get('cb_dont_' + string.replace(item['name'], ' ', '_'), 0):
                pList.append(item['name'])
        RESPONSE.setCookie('DontListProducts', pList, expires=d.rfc822(), path="/")
    if id!='Control_Panel':
        pList = []
        for item in self.unfiltered_meta_types():
            if REQUEST.get('cb_do_' + string.replace(item['name'], ' ', '_'), 0):
                pList.append(item['name'])
        RESPONSE.setCookie('DoListProducts', pList, expires=d.rfc822(), path="/")
    RESPONSE.redirect('%s/manage_hideProductsForm' % self.absolute_url())

targets = (
    App.ApplicationManager.ApplicationManager,
    OFS.Folder.Folder,
)
manage_options = (
    {'label':'Hide Products', 'action':'manage_hideProductsForm',
     'help':('PatchKit', 'ProductList.stx'),
    },
)
backup_methods = {
    'all_meta_types': 'unfiltered_meta_types',
}
methods = {
    'all_meta_types': all_meta_types,
    'globalExclude': getGlobalExclude,
    'personalExclude': getPersonalExclude,
    'personalInclude': getPersonalInclude,
    'manage_hideProductsForm':
        Globals.DTMLFile('www/HideProducts', globals()),
    'manage_hideProducts': manage_hideProducts,
}
