"""
    Patches products to provide a way to package FS products TTW

Thanks go to:
    Pavel Zaichenko <zaichep@mail.com>
        Patch to solve INSTANCE_HOME problem.
"""
__version__ = '4.1.0'

import os
import string
import App.Product
import Products
import Globals
import App.tar

class MyTGZ(App.tar.tgzarchive):
    """ Simple derived class that just changes the owner """
    def add(self, name, data):
        self._f.write(str(App.tar.TarEntry(name, data, uname="The", gname="Haqa")))

def manage_makePackage(self, REQUEST, RESPONSE, submit=None):
    """ Build a package tarball """
    ap, items = fileLists(self)
    tar = MyTGZ(self.productFilename(0))
    entries = []
    for item in items:
        item = os.path.normpath(item)
        file = open(os.path.join(ap, item))
        unixName = string.join(string.split(item, os.sep), '/')
        tar.add(unixName, file.read())
        file.close()
    tar.finish()
    data = tar.__str__()
    RESPONSE.setHeader('Content-Type', 'application/x-gzip-compressed; name="%s"' % self.productFilename(1))
    RESPONSE.setHeader('Content-Disposition', 'inline; filename="%s"' % self.productFilename(1))
    RESPONSE.setHeader('Content-Length', '%d' % len(data))
    RESPONSE.write(data)
    return ''

def _getFiles(folder, lead):
    """ Walk this folder """
    files = []
    for file in os.listdir(folder):
        if os.path.isdir(os.path.join(folder,file)):
            files.extend(_getFiles(os.path.join(folder, file),
                os.path.join(lead, file)))
        else:
            files.append(os.path.join(lead, file))
    return files

def fileLists(self):
    """ Return a tuple containing (included_files, excluded_files) """
    escape = "let_me_out"
    incl = []
    try:
        for folder in Products.__path__:
            productFolder = os.path.join(folder, self.id)
            if os.path.isdir(productFolder):
                for file in _getFiles(productFolder, self.id):
                    if os.path.splitext(file)[1] not in excludeExtensions:
                        incl.append(file)
                raise escape, (folder, incl)
    except escape, (f, i):
        incl.sort()
        return (f, i)
    return (None, [])
    
def productVersion(self):
    """ """
    version = '1.0.0'
    for productDir in Products.__path__:
        p = os.path.join(productDir, self.id, 'version.txt')
        if not os.path.exists(p):
            p = os.path.join(productDir, self.id, 'VERSION.txt')
        if not os.path.exists(p):
            p = os.path.join(productDir, self.id, 'VERSION.TXT')
        if os.path.exists(p):
            try:
                file = open(p)
                text = file.read()
                file.close()
                version = text
                break
            except:
                pass
    version = string.replace(version, '-', ' ')
    version = string.replace(version, '.', ' ')
    version = string.split(version, ' ')
    version.reverse()
    list = []
    for item in version:
        try:
            list.append("%d" % int(item))
        except:
            pass
    list.reverse()
    version = string.join(list, '.')
    return version

def productFilename(self, includeExtension=1):
    """ Return the filename for this tarball """
    name = "%s-%s" % (self.id, self.productVersion())
    if includeExtension:
        name = name + ".tgz"
    return name

def initializeProduct(productp, name, home, app):
    """ Replacement Product Initializer """
    target = App.Product.pkInitializeProduct(productp, name, home, app)
    newOptions = []
    if hasattr(target, 'manage_options'):
        for option in target.manage_options:
            if (not option.has_key('label') or
                option['label'] != 'Package'):
                newOptions.append(option)
        newOptions.append(
            {'label': 'Package', 'action': 'manage_makePackageForm',
             'help':('PatchKit', 'Package.stx'),
            }
        )
        target.manage_options = tuple(newOptions)
    return target

if not hasattr(App.Product, 'pkInitializeProduct'):
    App.Product.pkInitializeProduct = App.Product.initializeProduct
App.Product.initializeProduct = initializeProduct

excludeExtensions = (
    '.pyc',
    '.pyo',
    '.bak',
    '.py~',
    '.orig',
    '.old',
    '.swp',
    '.tmp',
)

targets = (
    App.Product.Product,
)
methods = {
    'manage_makePackageForm':   Globals.HTMLFile('www/Package', globals()),
    'manage_makePackage':       manage_makePackage,
    'excludeExtensions':        excludeExtensions,
    'fileLists':                fileLists,
    'productVersion':           productVersion,
    'productFilename':          productFilename,
}
